//-----------------------------------------------------------------
// SX15 con FLEX-Analogi. Firmware di test
// Versione : 1.0
// Area SX
// http://www.areasx.com
// Note     :
//-----------------------------------------------------------------

#memmap xmem

//-----------------------------------------------------------------
// Configurazione Firmware
//-----------------------------------------------------------------
#define DEBUG_INFO


//-----------------------------------------------------------------
// Define generali
//-----------------------------------------------------------------


//-----------------------------------------------------------------
// Define Hardware
//-----------------------------------------------------------------
#define LED1_ON       BitWrPortI(PFDR,&PFDRShadow,1,0)
#define LED1_OFF      BitWrPortI(PFDR,&PFDRShadow,0,0)
#define LED2_ON       BitWrPortI(PFDR,&PFDRShadow,1,1)
#define LED2_OFF      BitWrPortI(PFDR,&PFDRShadow,0,1)

#define KEY_PRESSED   BitRdPortI(PBDR,0)

// Output
#define STROBE        BitWrPortI(PFDR,&PFDRShadow,1,6);BitWrPortI(PFDR,&PFDRShadow,0,6)
#define CLOCK         BitWrPortI(PFDR,&PFDRShadow,1,4);BitWrPortI(PFDR,&PFDRShadow,0,4)
#define SET_DATA      BitWrPortI(PFDR,&PFDRShadow,1,7)
#define CLEAR_DATA    BitWrPortI(PFDR,&PFDRShadow,0,7)



// -----------------------------------------------------------------------
// Define I2C
// -----------------------------------------------------------------------
#define I2CSCLBit 7       //Linea di clock PB7
#define I2CSDABit 5       //Linea Data PB5

// Sposta il port dell'I2C sul PortE
#define i2c_SCL_H() BitWrPortI(PBDDR,&PBDDRShadow,0,I2CSCLBit)
#define i2c_SCL_L() BitWrPortI(PBDDR,&PBDDRShadow,1,I2CSCLBit)
#define i2c_SDA_H() BitWrPortI(PBDDR,&PBDDRShadow,0,I2CSDABit)
#define i2c_SDA_L() BitWrPortI(PBDDR,&PBDDRShadow,1,I2CSDABit)
#define i2c_SCL()   BitRdPortI(PBDR,I2CSCLBit)
#define i2c_SDA()   BitRdPortI(PBDR,I2CSDABit)

#define DS1621_ADDRESS 0x03     // Indirizzo del sensore di temperatura
#define AD1_ADDRESS 0x00        // Indirizzo del convertitore AD numero 1
#define AD2_ADDRESS 0x01        // Indirizzo del convertitore AD numero 2

#define COSTANTE (float)5/255   // Costante per la conversione AD


#use "I2C.LIB"


//-----------------------------------------------------------------
// Globali
//-----------------------------------------------------------------
char tmpbuf[1024];          // Buffer temporaneo

//-----------------------------------------------------------------
// Prototipi
//-----------------------------------------------------------------
void BoardInit(void);
nodebug void msDelay(unsigned int );
float DS1621Read(unsigned char);
int DS1621Init(unsigned char);
int PCF8591Read(unsigned char , char *, unsigned char );

//-----------------------------------------------------------------
// General purpose delay routine
//-----------------------------------------------------------------
nodebug void msDelay(unsigned int delay) {
  auto unsigned long tt;
  tt=MS_TIMER;
  while (MS_TIMER<delay+tt) ;

}

//--------------------------------------------------------
// Inizializza i port del modulo RCM3700
// Riceve      : -
// Restituisce : -
// Note        :
//--------------------------------------------------------
void BoardInit() {
  // Inizializzazione port PB0 in ingresso (pulsante)
  BitWrPortI(PBDDR, &PBDDRShadow, 0,0);
  // PB7 - I2C - SCL
  BitWrPortI(PBDDR, &PBDDRShadow, 0, 7);
  BitWrPortI(PBDR,&PBDRShadow,0,I2CSCLBit); //SCL Basso per generare il clock
  // PB5 - I2C - SDA (bidirezionale)
  BitWrPortI(PBDDR, &PBDDRShadow, 0, 5);
  BitWrPortI(PBDR,&PBDRShadow,0,I2CSDABit); //SDA Basso


  // Inizializzazione port PC0 (reset GSM)
  BitWrPortI(PCFR,&PCFRShadow,0,0);
  BitWrPortI(PCDR,&PCDRShadow,1,0);


  // Inizializzazione port PD4 (Power on control)
  BitWrPortI(PDFR, &PDFRShadow, 0, 4); // Set PD4 to be a normal I/O pin
  BitWrPortI(PDDDR, &PDDDRShadow, 0, 4); // Set PD4 to be an Input

  // PD5 - ICL (Input Clock)
  BitWrPortI(PDFR, &PDFRShadow, 0, 5); // Set PD4 to be a normal I/O pin
  BitWrPortI(PDDDR, &PDDDRShadow, 1, 5); // Set PD4 to be an Output
	// Abilita i CS e gli strobe
	WrPortI(PEDR, &PEDRShadow, 0x09);	// set PE0 and PE3 high
	WrPortI(PEDDR, &PEDDRShadow, 0x09);	// set PE0 and PE3 as outputs

	WrPortI(PEFR, &PEFRShadow, 0x01);	// enable I/O strobe on PE0
	WrPortI(IB0CR, &IB0CRShadow, 0x48);	// PE0 is a chip select strobe (era C8)

  // PE1 - IDA (Input Data)
  BitWrPortI(PEDDR,&PEDDRShadow,0,1);
  // PE4 - PL (Parallel Load)
  BitWrPortI(PEDDR,&PEDDRShadow,1,4);

  // Inizializzazione port PE7 (On/Off Modem)
  BitWrPortI(PEDDR, &PEDDRShadow, 1,7);


  // Inizializzazione seriale su PC2-PC3 (Ser C)
  BitWrPortI(PCFR, &PCFRShadow, 1,2);

  // Inizializzazione port PF0 PF1, PF4 (Led)
  WrPortI(PFFR, &PFFRShadow, 0x00);
  WrPortI(PFDCR, &PFDCRShadow, 0x00);
  BitWrPortI(PFDDR, &PFDDRShadow, 1,0);
  BitWrPortI(PFDDR, &PFDDRShadow, 1,1);
  // PF4 - OCL (Output Clock) - Shared with led
  BitWrPortI(PFDDR, &PFDDRShadow, 1,4);
  // PF6 - Strobe
  BitWrPortI(PFDDR, &PFDDRShadow, 1,6);
  // PF7 - ODA (Output Data)
  BitWrPortI(PFDDR, &PFDDRShadow, 1,7);

  // Inizializzazione PG6-PG7 (seriale E CallId e Seriale RF)
  BitWrPortI(PGFR, &PGFRShadow, 1,6);
  BitWrPortI(PGDDR, &PGDDRShadow, 0,7);
  BitWrPortI(PGDDR, &PGDDRShadow, 1,6);

  // Inizializzazione libreria I2C
  i2c_init();

  // Inizializzazione del sensore di temperatura
  DS1621Init(DS1621_ADDRESS);
}





// -----------------------------------------------------------------------
// Inizializzazione del sensore di temperatura
// Riceve     : address: indirizzo I2C del sensore
// Restituisce: 1 se tutto OK, <=0 se ci sono errori
// -----------------------------------------------------------------------
int DS1621Init(unsigned char address){

  i2c_stop_tx();

  // Invia lo start
  if (i2c_startw_tx()){
    i2c_stop_tx();
    return -1;
  }

  // Indirizza il chip e attende la risposta
  if (i2c_wr_wait(0x90 + (address<<1))){
    i2c_stop_tx();
    return -2;
  }

  // Parametri di configurazione del DS1621

  // Invia il comando ACh (Access Config)
  if (i2c_write_char(0xAC)){
    i2c_stop_tx();
    return -3;
  }

  // Invia il Dato 02h
  if (i2c_write_char(0x2)){
    i2c_stop_tx();
    return -4;
  }

  // Invia lo start di sincronismo
  if (i2c_startw_tx()) {
    i2c_stop_tx();
    return -5;
  }

  // Indirizza il chip e attende la risposta
  if (i2c_wr_wait(0x90 + (address<<1))) {
    i2c_stop_tx();
    return -6;
  }

  // Invia il comando EEh che avvia le conversioni
  if (i2c_write_char(0xEE)){
    i2c_stop_tx();
    return -7;
  }

  // Chiusura
  i2c_stop_tx();
  return 1;

}


// -----------------------------------------------------------------------
// Funzione di lettura temperatura dal DS1621
// Riceve     : address - Indirizzo I2C
// Restituisce: valore di temperatura letto (float) se tutto OK, un valore
//              <-100 se errore
// -----------------------------------------------------------------------
float DS1621Read(unsigned char address) {

  auto unsigned char cnt;
  auto float temp;

  i2c_stop_tx();
  // Invia lo start di sincronismo
  if (i2c_startw_tx()) {
    i2c_stop_tx();
    return -101;
  }
  // Indirizza il chip
  if (i2c_wr_wait(0x90 + (address<<1))) {
    i2c_stop_tx();
    return -102;
  }
  // Invia il comando AAh (lettura dell'ultima temperatura campionata)
  if (i2c_write_char(0xAA)){
    i2c_stop_tx();
    return -103;
  }
  // Invia lo start di sincronismo
  if (i2c_startw_tx()) {
    i2c_stop_tx();
    return -104;
  }
  // Indirizzo  + linea RW/R a 1
  if (i2c_wr_wait(0x90 + (address<<1) + 1)){
    i2c_stop_tx();
    return -105;
  }

  for (cnt=0;cnt<2;cnt++)
  {
    if (i2c_read_char(&tmpbuf[cnt])){
      i2c_stop_tx();
      return -106;
    }
    if (cnt==1){
      i2c_send_nak();
    } else {
      i2c_send_ack();
    }
  }

  // Tutto Ok
  i2c_stop_tx();

  // Converto in float
  temp=0;
  // Imposto il mezzo grado
  if (tmpbuf[1]==0x80) temp=0.5;

  if (tmpbuf[0] & 0x80) {
    // Temperatura negativa
    temp+=(!tmpbuf[0])+1;
    temp=-temp;
  } else {
    // Temperatura >0
    temp+=tmpbuf[0];
  }

  return (temp);
}


// -----------------------------------------------------------------------
// Funzione di lettura degli ingressi analogici
// Riceve      : address : indirizzo I2C del chip AD da leggere
//               buf : buffer che ricever i dati letti
//               len : lunghezza dei dati letti nel buffer
// Restituisce : -
// Note        :
// -----------------------------------------------------------------------
int PCF8591Read(unsigned char address, char *buf, unsigned char len) {

  	auto unsigned char cnt;
  	auto short int err;

  	i2c_stop_tx();
  	
  	// Invia lo START
  	if (err=i2c_startw_tx()){
   	 i2c_stop_tx();
   	 return -10;   //ritorna il codice di errore -10
  	}

  	// Scrive l'indirizzo I2C e attende la risposta ACK dal dispositivo
  	if (err=i2c_wr_wait(0x90 | (address<<1))){
   	 i2c_stop_tx();
   	 return -20;   //ritorna il codice di errore -20
  	}

  	// Parametri di configurazione del PCF8591 (Vedi datasheet)

  	//Invia il comando di configurazuine
  	if (err=i2c_write_char(0x04)){
    	i2c_stop_tx();
    	return -30;   //ritorna il codice di errore -30
  	}

  	// Stop
   i2c_stop_tx();

  // Ciclo di lettura
  //invia lo start di sincronismo
  if (err=i2c_startw_tx()) {
    i2c_stop_tx();
    return -10;   //ritorna il codice di errore -10
  }


  // Invia l'indirizzo  + linea RW/R a 1
  if (err=i2c_wr_wait(0x91 | (address<<1))){
		i2c_stop_tx();
		return -70;
  }

  // Effettua una prima lettura
	i2c_read_char(&buf[cnt]);
	i2c_send_ack();

	//Ciclo di lettura degli n byte
  for (cnt=0;cnt<len;cnt++) {
	  err=i2c_read_char(&buf[cnt]);
	  if (err){
   	  i2c_stop_tx();
	    return -80;
    }

    if (cnt==(len-1)){
   	  i2c_send_nak();
	  } else {
    	i2c_send_ack();
    }
	}

  // Stop
  i2c_stop_tx();

  return 1;
}



//-----------------------------------------------------------------
//-----------------------------------------------------------------
//    MAIN
//-----------------------------------------------------------------
//-----------------------------------------------------------------
main () {

  int i;
  char read_volt[4];
  float Temperature;

  // Inizializzazione Scheda
  BoardInit();


  #ifdef DEBUG_INFO
    printf ("System Ready\n");
  #endif


  while (1) {

    msDelay(1000);

    printf ("LETTURA TEMPERATURA -----------\n");
    Temperature=DS1621Read(DS1621_ADDRESS);
    printf ("Temperatura [%.1f]\n",Temperature);
    printf ("----------------------------------\n\n");


    printf ("LETTURA AD 1 --------------------\n");
    if(PCF8591Read(AD1_ADDRESS,read_volt,4) < 1) {
      // Segnala eventuali errori di lettura
      printf("Errore AD\n");
		} else {
		  	// Visualizza i valori letti
	   	for (i=0;i<4;i++) {
      		printf("Lettura Ch %d [%03d]:  V%.2f\n", i, read_volt[i], (read_volt[i] * COSTANTE));
	   	}
		}
    printf ("\nLETTURA AD 2 --------------------\n");
    if(PCF8591Read(AD2_ADDRESS,read_volt,4) < 1) {
      // Segnala eventuali errori di lettura
      printf("Errore AD\n");
		} else {
		  	// Visualizza i valori letti
	   	for (i=0;i<4;i++) {
      		printf("Lettura Ch %d [%03d]:  V%.2f\n", i, read_volt[i], (read_volt[i] * COSTANTE));
	   	}
		}
     printf("----------------------------------\n\n");

  }

}